#!/usr/bin/env python
# -*- coding: UTF8 -*-

# colligo.py 
#
#  Copyright (c) 2006 INdT (Instituto Nokia de Technologia)
#
#  Author: Kenneth Rohde Christiansen <kenneth.christiansen@gmail.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA

import pygtk
pygtk.require("2.0")
import gtk
import gobject
import tapioca
import sys
import os
import signal
import time


class ClientVideoChannel(object):

	def exit_program(self):
		self.connection.disconnect()
    		gtk.main_quit()
   		print "Disconnected. Finishing test..."


	def __init__(self):
		self.fixed = gtk.Fixed()

    		self.window = gtk.Window()
		self.window.set_title("Video Test")
		self.window.set_border_width(6)
		self.window.set_size_request(200, 200)

   		self.preview = gtk.DrawingArea()
    		self.preview.set_size_request(50, 50)

    		self.fixed.put(self.preview, 0, 0)
    		self.window.add(self.fixed)
    		self.window.show_all()

		tapioca.thread_init(False)

    		tapioca.debug_set_flags_from_env ()

    		self.factory = tapioca.ManagerFactory()

		print "..."

		if self.factory == None:
			print "failed to create Connection Manager Factory!"
			sys.exit(1)

		print "..."

    		self.manager = self.factory.get_manager("jabber")

		if self.manager == None:
			print "failed to create Connection Manager!"
			sys.exit(1)

		account = os.environ.get("ACCOUNT")
		password = os.environ.get("PASSWORD")

		if not account:
			print "please set environment ACCOUNT!\n"
			sys.exit(1)
		else:
			print "account:%s\n" % account

		if not password:
			print "please set environment PASSWORD!\n"
			sys.exit(1)
 		else:
			print "password:%s\n" % "**********"
	
    		parameters = self.manager.get_protocol_parameters("jabber")

		parameters.set_value_as_string("account", account)
		parameters.set_value_as_string("password", password)
		parameters.set_value_as_string("server", "talk.google.com")
		parameters.set_value_as_string("resource", "tapioca")
		parameters.set_value_as_uint("port", 5223)
		parameters.set_value_as_boolean("old-ssl", True)
		parameters.set_value_as_boolean("ignore-ssl-errors", True)
		
    		self.connection = self.manager.request_connection("jabber", parameters)

		if self.connection == None:
			print "failed to create Connection!"
			sys.exit(1)

		self.connection.connect("status-changed", self.status_changed_cb)
		self.connection.connect("channel-created", self.channel_created_cb)

		self.connection.make_connection()

		gtk.main()

		return 0

	def status_changed_cb(self, conn, status, reason):
		print "Got state-changed signal from TpaConnection\nstatus: %d, reason: %d\n" % (status, reason)

		if status == tapioca.CONNECTION_STATUS_CONNECTED:
			print "Connected!\nProtocol: %s\n" % conn.get_protocol()
			print "Status: %d\n" % conn.get_status()
			
			conn.get_open_channels()
			user = conn.get_user_contact()
			list = conn.get_contactlist()

			contact = list.get_contact(os.environ.get("CONTACT"))
			
			user.set_capabilities(tapioca.CAPABILITY_VIDEO|tapioca.CAPABILITY_AUDIO)
			conn.create_channel(tapioca.CHANNEL_TYPE_STREAM, contact)
    
		elif status == tapioca.CONNECTION_STATUS_DISCONNECTED:
			print "Disconnected. Finishing test..."
			gtk.main_quit()

	def stream_state_changed_cb(self, stream, state):
		if stream.get_stream_type() == tapioca.STREAM_TYPE_VIDEO and \
			state == tapioca.STREAM_STATE_PLAYING:
			print "Setting window"
	
			stream.set_window_id(self.window.window.xid)
		
			stream.add_preview_window_id(self.preview.window.xid)

	def stream_created_cb(self, channel, stream):
		stream.connect("state-changed", self.stream_state_changed_cb)

		if stream.get_stream_type == tapioca.STREAM_TYPE_AUDIO:
			print "New audio stream added!"
		elif stream.get_stream_type == tapioca.STREAM_TYPE_VIDEO:
			print "New video stream added!"

	def channel_created_cb(self, connection, channel):
		print "Got channel-created signal from TpaConnection\nchannel: %s\n" % channel

		if channel.get_channel_type() == tapioca.CHANNEL_TYPE_STREAM:
			stream_list = channel.get_streams()

			channel.connect("stream-added", self.stream_created_cb)

			if stream_list != None:
				iter = 0
				length = len(stream_list)
				while iter < length:
					self.stream_channel_created(channel, stream_list[iter])
					iter = iter + 1

			types = [tapioca.STREAM_TYPE_AUDIO, tapioca.STREAM_TYPE_VIDEO]

			channel.request_streams(channel.get_target(), types)

			if not channel.has_joined():
				channel.join()

if __name__ == "__main__": 
	client = ClientVideoChannel()

